import UIKit

class BaseViewController: UIViewController {

    private var rootViewOriginalFrame: CGRect = CGRect(x: 0, y: 0, width: 0, height: 0)

    override func viewDidLoad() {
        super.viewDidLoad()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        NotificationCenter.default.addObserver(self, selector: #selector(BaseViewController.keyboardWillShow(_:)), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(BaseViewController.keyboardWillHide(_:)), name: UIResponder.keyboardWillHideNotification, object: nil)
    }

    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)

        NotificationCenter.default.removeObserver(self)
    }

    func closeKeyboard() {
        self.keyboardWillHide(nil)
    }

    @objc func keyboardWillShow(_ aNotification: NSNotification) {
        let activeView = self.findActiveResponderView(self.view)
        if let forcusedView = activeView {
            self.slideUpKeyboardByNotification(aNotification, partsFrame: forcusedView.frame)
        }
    }

    @objc func keyboardWillHide(_ aNotification: NSNotification?) {
        self.slideDownKeyboardToOriginalPosition()
    }

    func findActiveResponderView(_ view: UIView) -> UIView? {
        if view.isFirstResponder {
            return view
        }
        for sub in view.subviews {
            let found = self.findActiveResponderView(sub)
            if let _ = found {
                return found
            }
        }
        return nil
    }

    @discardableResult
    func slideUpKeyboardByNotification(_ notification: NSNotification, partsFrame: CGRect) -> CGFloat {
        let targetView = self.view
        if let _ = targetView {
            let dict: NSDictionary? = notification.userInfo! as NSDictionary
            if let _ = dict {
                let s = dict![UIResponder.keyboardFrameEndUserInfoKey] as! NSValue
                let keyboardSize = s.cgRectValue
                let suggestionHeight = partsFrame.origin.y + partsFrame.size.height + UIApplication.shared.statusBarFrame.height + (self.navigationController?.navigationBar.frame.height)! - keyboardSize.origin.y
                let slidesize = targetView?.frame.origin.y == 0 ? suggestionHeight : self.view.frame.origin.y + suggestionHeight
                if slidesize <= 0 {
                    return 0.0
                }
                self.rootViewOriginalFrame = targetView!.frame
                UIView.animate(withDuration: 0.3, animations: {
                    targetView?.frame.origin.y -= slidesize
				})
                return slidesize
            }
        }
        return 0.0
    }

    func slideDownKeyboardToOriginalPosition() {

        if self.rootViewOriginalFrame.size.height == 0 {
            return
        }
        let targetView = self.view
        if let _ = targetView {
            if targetView?.frame.size.height != self.rootViewOriginalFrame.size.height {
                self.rootViewOriginalFrame.origin.y += (self.rootViewOriginalFrame.size.height - targetView!.frame.size.height)
            }
            self.rootViewOriginalFrame.origin = CGPoint.zero
            UIView.animate(
                withDuration: 0.3,
                animations: { () -> Void in
                    targetView!.frame = self.rootViewOriginalFrame
                },
                completion: { (_: Bool) in
                    self.rootViewOriginalFrame = CGRect(x: 0, y: 0, width: 0, height: 0)
                }
            )
        }
    }
}
