import Foundation
import mPOS2SDK

class LogData: NSObject {

    private var logText = ""

    func toString() -> String {
        return self.logText
    }

    func clear() {
        self.logText = ""
    }

    func addResponse(_ response: [ResponseParamValue]) {
        self.logText = LogData.stringFromResponseArray(response) + self.logText
    }

    private class func sortedParams(_ originalParams: [MposResponseParam: AnyObject], paramOrders: [MposResponseParam], checkExistance: Bool) -> [ResponseParamValue] {
        var restParams = originalParams

        // オーダー順に項目を追加
        var resultParams = paramOrders.reduce([ResponseParamValue]()) { result, param in
            var item: ResponseParamValue
            if let value = restParams[param] {
                item = ResponseParamValue(param, value, .ok, false)
            } else {
                if !checkExistance {
                    return result
                }
                item = ResponseParamValue(param, "" as AnyObject, .missing, false)
            }

            restParams.removeValue(forKey: param)
            return result + [item]
        }

        // 未使用の項目を追加
        resultParams += restParams.map { ResponseParamValue($0.0, $0.1, .unused, false) }

        return resultParams
    }

    class func sortedPaymentParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode,
            .orderId,
            .orderReferenceId,
            .paymentMethodCode,
            .paymentSource,
            .amount,
            .discountAmount,
            .totalAmount,
            .paymentTime,
            .orderDescription,
            .cardNumber,
            .cardNumberToken,
            .cardProviderCode,
            .cardExpiry,
            .cardType,
            .jpo,
            .authCode,
            .acquirerCode,
            .centerAcquirerId,
            .centerTradeId,
            .centerProcessId,
            .centerReferenceCode,
            .centerTransactionType,
            .centerTransactionDatetime,
            .centerTerminalId,
            .centerMerchantId,
            .centerMerchantName,
            .cupRequestTime,
            .settledAmount,
            .settledScale,
            .settledCurrency,
            .emvAppId,
            .emvAtc,
            .emvAppLabel,
            .panSeqNumber,
            .aosa,
            .cvmType,
            .cvmr,
            .tvr,
            .tsi,
            .txnCryptogram,
            .mileageMembershipNo,
            .usageType,
            .lastUseDate
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        return self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
    }

    class func sortedSearchParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode,
            .historyList
        ]

        let historyOrders: [MposResponseParam] = [
            .orderId,
            .orderReferenceId,
            .paymentMethodCategory,
            .paymentMethodCode,
            .paymentSource,
            .orderStatus,
            .amount,
            .discountAmount,
            .totalAmount,
            .registeredTime,
            .paymentTime,
            .status,
            .resultCode,
            .orderDescription,
            .cardNumber,
            .cardNumberToken,
            .cardProviderCode,
            .cardExpiry,
            .cardType,
            .jpo,
            .authCode,
            .acquirerCode,
            .centerAcquirerId,
            .centerTradeId,
            .centerProcessId,
            .centerReferenceCode,
            .centerTransactionType,
            .centerTransactionDatetime,
            .centerRefundDatetime,
            .centerTerminalId,
            .centerMerchantId,
            .centerMerchantName,
            .cupRequestTime,
            .settledAmount,
            .settledScale,
            .settledCurrency,
            .emvAppId,
            .emvAtc,
            .emvAppLabel,
            .panSeqNumber,
            .aosa,
            .cvmType,
            .cvmr,
            .tvr,
            .tsi,
            .txnCryptogram,
            .mileageMembershipNo,
            .usageType,
            .lastUseDate
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        let rootParams = self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
        if let historyPages = rootParams.first(where: { $0.param == .historyList }) {
            if let originalHistoryPages = historyPages.value as? [[MposResponseParam: AnyObject]] {
                historyPages.value = originalHistoryPages.map { sortedParams($0, paramOrders: historyOrders, checkExistance: checkExistance) }
            }
        }
        return rootParams
    }

    class func sortedMailParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        return self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
    }

    class func sortedRefundParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode,
            .orderId,
            .paymentMethodCode,
            .paymentSource,
            .amount,
            .discountAmount,
            .totalAmount,
            .cardNumber,
            .cardNumberToken,
            .cardProviderCode,
            .cardExpiry,
            .cardType,
            .jpo,
            .authCode,
            .acquirerCode,
            .centerAcquirerId,
            .centerTradeId,
            .centerProcessId,
            .centerReferenceCode,
            .centerTransactionType,
            .centerTransactionDatetime,
            .centerRefundDatetime,
            .centerTerminalId,
            .centerMerchantId,
            .centerMerchantName,
            .cupRequestTime,
            .emvAppId,
            .emvAtc,
            .emvAppLabel,
            .panSeqNumber,
            .aosa,
            .cvmType,
            .cvmr,
            .tvr,
            .tsi,
            .txnCryptogram,
            .mileageMembershipNo
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        return self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
    }

    class func sortedCancelParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode,
            .orderId,
            .paymentMethodCode,
            .jpo,
            .cardNumber,
            .cardProviderCode,
            .cardExpiry,
            .authCode,
            .acquirerCode
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        return self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
    }

    class func sortedPasswordChangeParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        return self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
    }

    class func sortedReportParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode,
            .reportTimeFrom,
            .reportTimeTo,
            .paymentMethodCode,
            .orderStatus,
            .salesReport,
            .mileageReport
        ]

        let salesOrders: [MposResponseParam] = [
            .captureTotalAmount,
            .refundTotalAmount,
            .differenceTotalAmount,
            .captureCount,
            .refundCount,
            .differenceCount
        ]

        let mileageOrders: [MposResponseParam] = [
            .accrualTotalAmount,
            .cancelTotalAmount,
            .differenceTotalAmount,
            .accrualCount,
            .cancelCount,
            .differenceCount
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        let rootParams = self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
        if let salesReports = rootParams.first(where: { $0.param == .salesReport }) {
            if let originalSalesReports = salesReports.value as? [MposResponseParam: AnyObject] {
                salesReports.value = self.sortedParams(originalSalesReports, paramOrders: salesOrders, checkExistance: checkExistance)
                salesReports.isBraces = true
            }
        }
        if let mileageReports = rootParams.first(where: { $0.param == .mileageReport }) {
            if let originalMileageReports = mileageReports.value as? [MposResponseParam: AnyObject] {
                mileageReports.value = self.sortedParams(originalMileageReports, paramOrders: mileageOrders, checkExistance: checkExistance)
                mileageReports.isBraces = true
            }
        }
        return rootParams
    }

    class func sortedUpdateBTParams(_ response: [MposResponseParam: AnyObject]) -> [ResponseParamValue] {
        let orders: [MposResponseParam] = [
            .status,
            .errorCode,
            .errorMessage,
            .resultCode
        ]

        let checkExistance = (response[.status] as? String ?? "") == "success"
        return self.sortedParams(response, paramOrders: orders, checkExistance: checkExistance)
    }

    class func stringFromResponseObject(_ obj: ResponseParamValue) -> String {
        return self.stringFromResponseElement(obj)
    }

    class func stringFromResponseArray(_ array: [ResponseParamValue]) -> String {
        var text = ""
        for item in array {
            text += self.stringFromResponseElement(item)
        }
        return text
    }

    private class func stringFromResponseElement(_ obj: Any) -> String {
        if let text = obj as? String {
            return text
        }

        if let item = obj as? ResponseParamValue {
            var text = ""
            text += item.param.rawValue
            text += ": "

            if item.isBraces {
                text += "{"
            }

            text += self.stringFromResponseElement(item.value)
            if !(item.value is [AnyObject]) {
                text += "\n"
            }

            if item.isBraces {
                text += "}\n"
            }

            return text
        }

        if let array = obj as? [ResponseParamValue] {
            var text = ""
            for item in array {
                text += self.stringFromResponseElement(item)
            }
            return text
        }

        if let array = obj as? [AnyObject] {
            var text = "[\n"
            var isFirst = true
            for item in array {
                if !isFirst {
                    text += ", "
                }
                text += self.stringFromResponseElement(item)

                isFirst = false
            }
            text += "]\n"
            return text
        }

        return ""
    }
}
