import mPOS2SDK
import UIKit

class MailViewController: UIViewController {

    private let logData = LogData()
    private var textFieldController: TextFieldController?

    @IBOutlet weak var orderIdSwitch: UISwitch!
    @IBOutlet weak var orderIdTextField: UITextField!
    @IBOutlet weak var emailSwitch: UISwitch!
    @IBOutlet weak var emailTextField: UITextField!
    @IBOutlet weak var mailTypeSwitch: UISwitch!
    @IBOutlet weak var mailTypeTextField: UITextField!
    @IBOutlet weak var coverView: UIView!

    let handler = MposHandler()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.navigationItem.rightBarButtonItem = UIBarButtonItem.logButtonItemWithTarget(self, action: #selector(MailViewController.onLogButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.orderIdTextField.delegate = self.textFieldController
        self.emailTextField.delegate = self.textFieldController
        self.mailTypeTextField.delegate = self.textFieldController

        self.coverView.isHidden = true
    }

    @objc func onLogButton(_ sender: AnyObject) {
        self.navigationController?.pushViewController(LogViewController.logViewControllerWithOwnerViewController(self, logData: self.logData), animated: true)
    }

    @IBAction func onSendButton(_ sender: AnyObject) {

        var requestParam = [MposRequestParam: String]()

        if self.orderIdSwitch.isOn {
            requestParam[MposRequestParam.orderId] = self.orderIdTextField.text
        }

        if self.emailSwitch.isOn {
            requestParam[MposRequestParam.email] = self.emailTextField.text
        }

        if self.mailTypeSwitch.isOn {
            requestParam[MposRequestParam.mailType] = self.mailTypeTextField.text
        }

        self.coverView.isHidden = false

        self.handler.requestSendReceipt(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            self.coverView.isHidden = true

            let sortedParams = LogData.sortedMailParams(response)
            self.logData.addResponse(sortedParams)
            self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
		})
    }
}
