import mPOS2SDK
import UIKit

class PasswordViewController: UIViewController {

    private let logData = LogData()
    private var textFieldController: TextFieldController?

    @IBOutlet weak var oldPasswordSwitch: UISwitch!
    @IBOutlet weak var oldPasswordTextField: UITextField!
    @IBOutlet weak var nextPasswordSwitch: UISwitch!
    @IBOutlet weak var nextPasswordTextField: UITextField!
    @IBOutlet weak var coverView: UIView!

    let handler = MposHandler()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.navigationItem.rightBarButtonItem = UIBarButtonItem.logButtonItemWithTarget(self, action: #selector(PasswordViewController.onLogButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.oldPasswordTextField.delegate = self.textFieldController
        self.nextPasswordTextField.delegate = self.textFieldController

        self.coverView.isHidden = true
    }

    @objc func onLogButton(_ sender: AnyObject) {
        self.navigationController?.pushViewController(LogViewController.logViewControllerWithOwnerViewController(self, logData: self.logData), animated: true)
    }

    @IBAction func onChangeButton(_ sender: AnyObject) {

        var requestParam = [MposRequestParam: String]()

        if self.oldPasswordSwitch.isOn {
            requestParam[MposRequestParam.oldPasswd] = self.oldPasswordTextField.text
        }

        if self.nextPasswordSwitch.isOn {
            requestParam[MposRequestParam.newPasswd] = self.nextPasswordTextField.text
        }

        self.coverView.isHidden = false

        self.handler.requestChangePassword(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            self.coverView.isHidden = true

            let sortedParams = LogData.sortedPasswordChangeParams(response)
            self.logData.addResponse(sortedParams)
            self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
		})
    }
}
