import mPOS2SDK
import UIKit

class PaymentViewController: BaseViewController {

    private let logData = LogData()
    private var textFieldController: TextFieldController?

    @IBOutlet weak var amountTextField: UITextField!
    @IBOutlet weak var descriptionTextField: UITextField!
    @IBOutlet weak var descriptionSwitch: UISwitch!
    @IBOutlet weak var displayInputAmountPageSwitch: UISwitch!
    @IBOutlet weak var displayInputAmountPageTextField: UITextField!
    @IBOutlet weak var displayCompletePageSwitch: UISwitch!
    @IBOutlet weak var displayCompletePageTextField: UITextField!
    @IBOutlet weak var paymentMethodCodeSwitch: UISwitch!
    @IBOutlet weak var paymentMethodCodeTextField: UITextField!

    var handler = MposHandler()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.navigationItem.rightBarButtonItem = UIBarButtonItem.logButtonItemWithTarget(self, action: #selector(PaymentViewController.onLogButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.amountTextField.delegate = self.textFieldController
        self.descriptionTextField.delegate = self.textFieldController
        self.displayInputAmountPageTextField.delegate = self.textFieldController
        self.displayCompletePageTextField.delegate = self.textFieldController
        self.paymentMethodCodeTextField.delegate = self.textFieldController
    }

    @objc func onLogButton(_ sender: AnyObject) {
        self.navigationController?.pushViewController(LogViewController.logViewControllerWithOwnerViewController(self, logData: self.logData), animated: true)
    }

    @IBAction func onPayButton(_ sender: AnyObject) {

        var requestParam = [MposRequestParam: String]()

        requestParam[MposRequestParam.amount] = self.amountTextField.text

        if self.descriptionSwitch.isOn {
            requestParam[MposRequestParam.orderDescription] = self.descriptionTextField.text
        }

        if self.displayInputAmountPageSwitch.isOn {
            requestParam[MposRequestParam.inputAmountView] = self.displayInputAmountPageTextField.text
        }

        if self.displayCompletePageSwitch.isOn {
            requestParam[MposRequestParam.completeView] = self.displayCompletePageTextField.text
        }

        if self.paymentMethodCodeSwitch.isOn {
            requestParam[MposRequestParam.paymentMethodCode] = self.paymentMethodCodeTextField.text
        }

        self.handler.startPayment(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            let sortedParams = LogData.sortedPaymentParams(response)
            self.logData.addResponse(sortedParams)
            self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
		})
    }
}
