import Foundation
import mPOS2SDK

class ReaderAppUpdateBTViewController: UIViewController {

    private let logData = LogData()
    private var textFieldController: TextFieldController?

    @IBOutlet weak var progressDialogSwitch: UISwitch!
    @IBOutlet weak var progressDialogTextField: UITextField!
    @IBOutlet weak var btConnectTimeoutSwitch: UISwitch!
    @IBOutlet weak var btConnectTimeoutTextField: UITextField!
    @IBOutlet weak var coverView: UIView!

    let handler = MposHandler()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.navigationItem.rightBarButtonItem = UIBarButtonItem.logButtonItemWithTarget(self, action: #selector(ReaderAppUpdateBTViewController.onLogButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.progressDialogTextField.delegate = self.textFieldController
        self.btConnectTimeoutTextField.delegate = self.textFieldController

        self.coverView.isHidden = true
    }

    @objc func onLogButton(_ sender: AnyObject) {
        self.navigationController?.pushViewController(LogViewController.logViewControllerWithOwnerViewController(self, logData: self.logData), animated: true)
    }

    @IBAction func onStartButton(_ sender: AnyObject) {

        var requestParam = [MposRequestParam: String]()

        if self.progressDialogSwitch.isOn {
            requestParam[MposRequestParam.progressDialog] = self.progressDialogTextField.text
        }

        if self.btConnectTimeoutSwitch.isOn {
            requestParam[MposRequestParam.btConnectTimeout] = self.btConnectTimeoutTextField.text
        }

        self.coverView.isHidden = false

        self.handler.requestReaderAppUpdateBT(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            self.coverView.isHidden = true

            let sortedParams = LogData.sortedUpdateBTParams(response)
            self.logData.addResponse(sortedParams)
            self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
        })
    }
}
