import mPOS2SDK
import UIKit

class RefundViewController: UIViewController {
    private let logData = LogData()
    private var textFieldController: TextFieldController?

    @IBOutlet weak var orderIdSwitch: UISwitch!
    @IBOutlet weak var orderIdTextField: UITextField!
    @IBOutlet weak var coverView: UIView!

    var handler = MposHandler()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.navigationItem.rightBarButtonItem = UIBarButtonItem.logButtonItemWithTarget(self, action: #selector(RefundViewController.onLogButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.orderIdTextField.delegate = self.textFieldController

        self.coverView.isHidden = true
    }

    @objc func onLogButton(_ sender: AnyObject) {
        self.navigationController?.pushViewController(LogViewController.logViewControllerWithOwnerViewController(self, logData: self.logData), animated: true)
    }

    @IBAction func onRefundButton(_ sender: AnyObject) {

        var requestParam = [MposRequestParam: String]()

        if self.orderIdSwitch.isOn {
            requestParam[MposRequestParam.orderId] = self.orderIdTextField.text
        }

        self.coverView.isHidden = false

        self.handler.requestRefund(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            self.coverView.isHidden = true

            let sortedParams = LogData.sortedRefundParams(response)
            self.logData.addResponse(sortedParams)
            self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
		})
    }

    @IBAction func onCancelButton(_ sender: Any) {
        var requestParam = [MposRequestParam: String]()

        if self.orderIdSwitch.isOn {
            requestParam[MposRequestParam.orderId] = self.orderIdTextField.text
        }

        self.coverView.isHidden = false

        self.handler.requestCancel(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            self.coverView.isHidden = true

            let sortedParams = LogData.sortedCancelParams(response)
            self.logData.addResponse(sortedParams)
            self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
        })
    }
}
