import Foundation
import mPOS2SDK

class ReportViewController: UIViewController {

    private let logData = LogData()
    private var textFieldController: TextFieldController?

    @IBOutlet weak var reportTimeFromSwitch: UISwitch!
    @IBOutlet weak var reportTimeFromTextField: UITextField!
    @IBOutlet weak var reportTimeToSwitch: UISwitch!
    @IBOutlet weak var reportTimeToTextField: UITextField!
    @IBOutlet weak var paymentMethodCodeSwitch: UISwitch!
    @IBOutlet weak var paymentMethodCodeTextField: UITextField!
    @IBOutlet weak var orderStatusSwitch: UISwitch!
    @IBOutlet weak var orderStatusTextField: UITextField!
    @IBOutlet weak var coverView: UIView!

    let handler = MposHandler()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.navigationItem.rightBarButtonItem = UIBarButtonItem.logButtonItemWithTarget(self, action: #selector(ReportViewController.onLogButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.reportTimeFromTextField.delegate = self.textFieldController
        self.reportTimeToTextField.delegate = self.textFieldController
        self.paymentMethodCodeTextField.delegate = self.textFieldController
        self.orderStatusTextField.delegate = self.textFieldController

        self.coverView.isHidden = true
    }

    @objc func onLogButton(_ sender: AnyObject) {
        self.navigationController?.pushViewController(LogViewController.logViewControllerWithOwnerViewController(self, logData: self.logData), animated: true)
    }

    @IBAction func onStartButton(_ sender: AnyObject) {

        var requestParam = [MposRequestParam: String]()

        if self.reportTimeFromSwitch.isOn {
            requestParam[MposRequestParam.reportTimeFrom] = self.reportTimeFromTextField.text
        }

        if self.reportTimeToSwitch.isOn {
            requestParam[MposRequestParam.reportTimeTo] = self.reportTimeToTextField.text
        }

        if self.paymentMethodCodeSwitch.isOn {
            requestParam[MposRequestParam.paymentMethodCode] = self.paymentMethodCodeTextField.text
        }

        if self.orderStatusSwitch.isOn {
            requestParam[MposRequestParam.orderStatus] = self.orderStatusTextField.text
        }

        self.coverView.isHidden = false

        self.handler.requestReport(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            self.coverView.isHidden = true

            let sortedParams = LogData.sortedReportParams(response)
            self.logData.addResponse(sortedParams)
            self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
        })
    }
}
