import mPOS2SDK
import UIKit

class SearchResultViewController: UITableViewController {

    private var params: [ResponseParamValue] = [] {
        didSet {
            if let historyList = params.first(where: { $0.param == .historyList }) {
                if let histories = historyList.value as? [[ResponseParamValue]] {
                    self.histories = histories
                }
            }
        }
    }

    private var histories = [[ResponseParamValue]]()

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.histories.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "SearchCell") as! SearchCell
        let items = self.histories[indexPath.row]

        cell.pageLabel.text = String(indexPath.row + 1)

        cell.itemViews.forEach {
            $0.isHidden = true
            $0.titleLabel.text = ""
            $0.valueLabel.text = ""
        }

        let normalColor = UIColor(red: 85 / 255.0, green: 85 / 255.0, blue: 85 / 255.0, alpha: 1.0)
        for (index, item) in items.enumerated() {
            let itemView = cell.itemViews[index]
            itemView.isHidden = false
            itemView.titleLabel.text = item.param.rawValue
            itemView.valueLabel.text = item.value as? String ?? ""

            var color: UIColor
            switch item.status {
            case .ok: color = normalColor
            case .missing: color = UIColor.brown
            case .unused: color = UIColor.blue
            }
            itemView.titleLabel.textColor = color
            itemView.valueLabel.textColor = color
        }

        return cell
    }
}

extension SearchResultViewController {

    class func searchResultViewControllerWithOwnerViewController(_ ownerViewController: UIViewController, params: [ResponseParamValue]) -> SearchResultViewController {
        let viewController = ownerViewController.storyboard?.instantiateViewController(withIdentifier: "SearchResultViewController") as! SearchResultViewController
        viewController.params = params
        return viewController
    }
}
