import mPOS2SDK
import UIKit

class SearchViewController: BaseViewController {
    private let logData = LogData()
    private var textFieldController: TextFieldController?

    @IBOutlet weak var transactionCountTextField: UITextField!
    @IBOutlet weak var orderReferenceIdSwitch: UISwitch!
    @IBOutlet weak var orderReferenceIdTextField: UITextField!
    @IBOutlet weak var orderIdSwitch: UISwitch!
    @IBOutlet weak var orderIdTextField: UITextField!
    @IBOutlet weak var registeredTimeFromSwitch: UISwitch!
    @IBOutlet weak var registeredTimeFromTextField: UITextField!
    @IBOutlet weak var registeredTimeToSwitch: UISwitch!
    @IBOutlet weak var registeredTimeToTextField: UITextField!
    @IBOutlet weak var coverView: UIView!

    var handler = MposHandler()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.navigationItem.rightBarButtonItem = UIBarButtonItem.logButtonItemWithTarget(self, action: #selector(SearchViewController.onLogButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.transactionCountTextField.delegate = self.textFieldController
        self.orderReferenceIdTextField.delegate = self.textFieldController
        self.orderIdTextField.delegate = self.textFieldController
        self.registeredTimeFromTextField.delegate = self.textFieldController
        self.registeredTimeToTextField.delegate = self.textFieldController

        self.coverView.isHidden = true
    }

    @objc func onLogButton(_ sender: AnyObject) {
        self.navigationController?.pushViewController(LogViewController.logViewControllerWithOwnerViewController(self, logData: self.logData), animated: true)
    }

    @IBAction func onSearchButton(_ sender: AnyObject) {

        var requestParam = [MposRequestParam: String]()

        requestParam[MposRequestParam.transactionCount] = self.transactionCountTextField.text

        if self.orderReferenceIdSwitch.isOn {
            requestParam[MposRequestParam.orderReferenceId] = self.orderReferenceIdTextField.text
        }

        if self.orderIdSwitch.isOn {
            requestParam[MposRequestParam.orderId] = self.orderIdTextField.text
        }

        if self.registeredTimeFromSwitch.isOn {
            requestParam[MposRequestParam.registeredTimeFrom] = self.registeredTimeFromTextField.text
        }

        if self.registeredTimeToSwitch.isOn {
            requestParam[MposRequestParam.registeredTimeTo] = self.registeredTimeToTextField.text
        }

        self.coverView.isHidden = false

        self.handler.requestSearchTransactionHistory(SettingData.requestHeader(), requestParams: requestParam, complete: { [unowned self] (response) -> Void in
            self.coverView.isHidden = true

            let sortedParams = LogData.sortedSearchParams(response)

            if let status = response[MposResponseParam.status] as? String, status == "failure" {
                self.present(UIAlertController.alertControllerWithResponse(sortedParams), animated: true, completion: { () -> Void in })
            } else {
                self.logData.addResponse(sortedParams)
                self.navigationController?.pushViewController(SearchResultViewController.searchResultViewControllerWithOwnerViewController(self, params: sortedParams), animated: false)
            }
		})
    }
}
