import Foundation
import mPOS2SDK
import UIKit

@objc
class SettingData: NSObject, NSCoding {

    private static let SaveDataKey = "SettingData"

    var userID = ""
    var userPassword = ""
    var merchantPassword = ""
    var readerType = "iPOS"
    var appNameType = "sdksample"
    var appVersion = "1.0"
    var locale = "ja"
    var cardThreeWay = ""
    var cameraFacing = ""
    var isUserPassword = NSNumber(value: true)
    var isMerchantPassword = NSNumber(value: true)
    var isReaderType = NSNumber(value: true)
    var isCardThreeWay = NSNumber(value: true)
    var isCameraFacing = NSNumber(value: true)

    private override init() {}

    class func settingData() -> SettingData {

        if let data = UserDefaults.standard.object(forKey: SettingData.SaveDataKey) as? NSData {
            return NSKeyedUnarchiver.unarchiveObject(with: (data as NSData) as Data) as! SettingData
        } else {
            let newSaveData = SettingData()
            newSaveData.save()
            return newSaveData
        }
    }

    func save() {
        UserDefaults.standard.set(NSKeyedArchiver.archivedData(withRootObject: self), forKey: SettingData.SaveDataKey)
        UserDefaults.standard.synchronize()
    }

    public func encode(with aCoder: NSCoder) {
        aCoder.encode(self.userID, forKey: "userID")
        aCoder.encode(self.userPassword, forKey: "userPassword")
        aCoder.encode(self.merchantPassword, forKey: "merchantPassword")
        aCoder.encode(self.readerType, forKey: "readerType")
        aCoder.encode(self.appNameType, forKey: "appNameType")
        aCoder.encode(self.appVersion, forKey: "appVersion")
        aCoder.encode(self.locale, forKey: "locale")
        aCoder.encode(self.cardThreeWay, forKey: "cardThreeWay")
        aCoder.encode(self.cameraFacing, forKey: "cameraFacing")
        aCoder.encode(self.isUserPassword, forKey: "isUserPassword")
        aCoder.encode(self.isMerchantPassword, forKey: "isMerchantPassword")
        aCoder.encode(self.isReaderType, forKey: "isReaderType")
        aCoder.encode(self.isCardThreeWay, forKey: "isCardThreeWay")
        aCoder.encode(self.isCameraFacing, forKey: "isCameraFacing")
    }

    required init?(coder aDecoder: NSCoder) {

        self.userID = aDecoder.decodeObject(forKey: "userID") as? String ?? ""
        self.userPassword = aDecoder.decodeObject(forKey: "userPassword") as? String ?? ""
        self.merchantPassword = aDecoder.decodeObject(forKey: "merchantPassword") as? String ?? ""
        self.readerType = aDecoder.decodeObject(forKey: "readerType") as? String ?? ""
        self.appNameType = aDecoder.decodeObject(forKey: "appNameType") as? String ?? ""
        self.appVersion = aDecoder.decodeObject(forKey: "appVersion") as? String ?? ""
        self.locale = aDecoder.decodeObject(forKey: "locale") as? String ?? ""
        self.cardThreeWay = aDecoder.decodeObject(forKey: "cardThreeWay") as? String ?? ""
        self.cameraFacing = aDecoder.decodeObject(forKey: "cameraFacing") as? String ?? ""
        self.isUserPassword = aDecoder.decodeObject(forKey: "isUserPassword") as? NSNumber ?? NSNumber(value: true)
        self.isMerchantPassword = aDecoder.decodeObject(forKey: "isMerchantPassword") as? NSNumber ?? NSNumber(value: true)
        self.isReaderType = aDecoder.decodeObject(forKey: "isReaderType") as? NSNumber ?? NSNumber(value: true)
        self.isCardThreeWay = aDecoder.decodeObject(forKey: "isCardThreeWay") as? NSNumber ?? NSNumber(value: true)
        self.isCameraFacing = aDecoder.decodeObject(forKey: "isCameraFacing") as? NSNumber ?? NSNumber(value: true)
    }
}

extension SettingData {
    class func requestHeader() -> [MposRequestParam: String] {
        let settingData = self.settingData()
        var headerParam = [MposRequestParam: String]()

        headerParam[MposRequestParam.userId] = settingData.userID

        if settingData.isUserPassword.boolValue {
            headerParam[MposRequestParam.userPassword] = settingData.userPassword
        } else {
            if settingData.isMerchantPassword.boolValue {
                headerParam[MposRequestParam.merchantPassword] = settingData.merchantPassword
            }
        }

        if settingData.isReaderType.boolValue {
            headerParam[MposRequestParam.readerType] = settingData.readerType
        }

        headerParam[MposRequestParam.locale] = settingData.locale

        if settingData.isCardThreeWay.boolValue {
            headerParam[MposRequestParam.cardThreeWay] = settingData.cardThreeWay
        }

        if settingData.isCameraFacing.boolValue {
            headerParam[MposRequestParam.cameraFacing] = settingData.cameraFacing
        }

        return headerParam
    }
}
