import UIKit

class SettingViewController: BaseViewController {

    private var textFieldController: TextFieldController?

    @IBOutlet weak var userIdTextField: UITextField!
    @IBOutlet weak var userPasswordSwitch: UISwitch!
    @IBOutlet weak var userPasswordTextField: UITextField!
    @IBOutlet weak var merchantPasswordSwitch: UISwitch!
    @IBOutlet weak var merchantPasswordTextField: UITextField!
    @IBOutlet weak var readerTypeSwitch: UISwitch!
    @IBOutlet weak var readerTypeTextField: UITextField!
    @IBOutlet weak var appNameTextField: UITextField!
    @IBOutlet weak var appVersionTextField: UITextField!
    @IBOutlet weak var localTextField: UITextField!
    @IBOutlet weak var cardThreeWaySwitch: UISwitch!
    @IBOutlet weak var cardThreeWayTextField: UITextField!
    @IBOutlet weak var cameraFacingSwitch: UISwitch!
    @IBOutlet weak var cameraFacingTextField: UITextField!

    override func viewDidLoad() {
        super.viewDidLoad()
        self.navigationItem.rightBarButtonItem = UIBarButtonItem.saveButtonItemWithTarget(self, action: #selector(SettingViewController.onSaveButton(_:)))

        self.textFieldController = TextFieldController.textFieldController(self, didChange: { (_) -> Void in })
        self.userIdTextField.delegate = self.textFieldController
        self.userPasswordTextField.delegate = self.textFieldController
        self.merchantPasswordTextField.delegate = self.textFieldController
        self.readerTypeTextField.delegate = self.textFieldController
        self.appNameTextField.delegate = self.textFieldController
        self.appVersionTextField.delegate = self.textFieldController
        self.localTextField.delegate = self.textFieldController
        self.cardThreeWayTextField.delegate = self.textFieldController
        self.cameraFacingTextField.delegate = self.textFieldController

        let settingData = SettingData.settingData()
        self.userIdTextField.text = settingData.userID
        self.userPasswordTextField.text = settingData.userPassword
        self.merchantPasswordTextField.text = settingData.merchantPassword
        self.readerTypeTextField.text = settingData.readerType
        self.appNameTextField.text = settingData.appNameType
        self.appVersionTextField.text = settingData.appVersion
        self.localTextField.text = settingData.locale
        self.cardThreeWayTextField.text = settingData.cardThreeWay
        self.cameraFacingTextField.text = settingData.cameraFacing
        self.userPasswordSwitch.isOn = settingData.isUserPassword.boolValue
        self.merchantPasswordSwitch.isOn = settingData.isMerchantPassword.boolValue
        self.readerTypeSwitch.isOn = settingData.isReaderType.boolValue
        self.cardThreeWaySwitch.isOn = settingData.isCardThreeWay.boolValue
        self.cameraFacingSwitch.isOn = settingData.isCameraFacing.boolValue
    }

    @objc func onSaveButton(_ sender: AnyObject) {
        self.doSave()

        if let controller = self.navigationController {
            controller.popViewController(animated: true)
        }
    }

    fileprivate func doSave() {
        let settingData = SettingData.settingData()
        settingData.userID = self.userIdTextField.text!
        settingData.userPassword = self.userPasswordTextField.text!
        settingData.merchantPassword = self.merchantPasswordTextField.text!
        settingData.readerType = self.readerTypeTextField.text!
        settingData.appNameType = self.appNameTextField.text!
        settingData.appVersion = self.appVersionTextField.text!
        settingData.locale = self.localTextField.text!
        settingData.cardThreeWay = self.cardThreeWayTextField.text!
        settingData.cameraFacing = self.cameraFacingTextField.text!
        settingData.isUserPassword = NSNumber(value: self.userPasswordSwitch.isOn)
        settingData.isMerchantPassword = NSNumber(value: self.merchantPasswordSwitch.isOn)
        settingData.isReaderType = NSNumber(value: self.readerTypeSwitch.isOn)
        settingData.isCardThreeWay = NSNumber(value: self.cardThreeWaySwitch.isOn)
        settingData.isCameraFacing = NSNumber(value: self.cameraFacingSwitch.isOn)
        settingData.save()
    }
}
