import UIKit

class TextFieldController: NSObject, UITextFieldDelegate {

    private weak var ownerViewController: UIViewController?
    private var didChange: ((String) -> Void)?
    private var tapGesture: UITapGestureRecognizer?
    private var textField: UITextField?

    class func textFieldController(_ ownerViewController: UIViewController, didChange: @escaping (String) -> Void) -> TextFieldController {
        let textFieldController = TextFieldController()
        textFieldController.ownerViewController = ownerViewController
        textFieldController.didChange = didChange
        textFieldController.tapGesture = UITapGestureRecognizer(target: textFieldController, action: #selector(TextFieldController.didTapInTextField(_:)))
        textFieldController.tapGesture!.numberOfTapsRequired = 1
        return textFieldController
    }

    func textFieldEditingChanged(_ textField: UITextField) {
        self.didChange!(textField.text!)
    }

    func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
        self.textField = textField
        self.ownerViewController!.view.addGestureRecognizer(self.tapGesture!)
        return true
    }

    func textFieldDidEndEditing(_ textField: UITextField) {
        self.ownerViewController!.view.removeGestureRecognizer(self.tapGesture!)
    }

    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        if textField.returnKeyType != UIReturnKeyType.next {
            textField.resignFirstResponder()
        }

        return true
    }

    @objc func didTapInTextField(_ sender: AnyObject) {
        self.textField!.resignFirstResponder()
    }
}
