import UIKit

extension UIAlertController {

    class func alertControllerWithLogMessage(_ log: String) -> UIAlertController {
        let alert = UIAlertController(title: "結果", message: "", preferredStyle: UIAlertController.Style.alert)

        let action = UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: { _ in })
        alert.addAction(action)

        let paragraphStyle = NSMutableParagraphStyle()
        paragraphStyle.alignment = NSTextAlignment.left
        let message = NSMutableAttributedString(string: log, attributes: [.paragraphStyle: paragraphStyle])

        alert.setValue(message, forKey: "attributedMessage")

        return alert
    }

    class func alertControllerWithResponse(_ response: [ResponseParamValue]) -> UIAlertController {
        let alert = UIAlertController(title: "結果", message: "", preferredStyle: UIAlertController.Style.alert)

        let action = UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: { _ in })
        alert.addAction(action)

        let message = NSMutableAttributedString(string: "")
        for item in response {
            var color: UIColor
            switch item.status {
            case .ok: color = .black
            case .missing: color = .brown
            case .unused: color = .blue
            }
            let attrString = NSAttributedString(string: LogData.stringFromResponseObject(item), attributes: [NSAttributedString.Key.foregroundColor: color])
            message.append(attrString)
        }

        // 左寄せ
        let paragraphStyle = NSMutableParagraphStyle()
        paragraphStyle.alignment = NSTextAlignment.left
        message.addAttribute(.paragraphStyle, value: paragraphStyle, range: NSMakeRange(0, message.length))

        alert.setValue(message, forKey: "attributedMessage")

        return alert
    }
}
